@@ -22,13 +22,20 @@ public class A3Panel extends JPanel implements ActionListener, ChangeListener {
     JMenuItem snapshotMenuItem;
     JMenuItem backgroundMenuItem;
     JMenuItem headLightMenuItem;
+    JMenu balloonMenu;
     JMenuItem balloonRightMenuItem;
     JMenuItem balloonLeftMenuItem;
     JMenuItem balloonTopMenuItem;
     JMenuItem balloonBottomMenuItem;
     JMenuItem balloonDelMenuItem;
+    JMenu labelMenu;
     JMenuItem labelMenuItem;
     JMenuItem labelDelMenuItem;
+    JMenu measureMenu;
+    JMenuItem measureXZMenuItem;
+    JMenuItem measureXYMenuItem;
+    JMenuItem measureYZMenuItem;
+    JMenuItem measureNoneMenuItem;
     JMenuItem htmlMenuItem;
     JMenuItem metadataMenuItem;
     JMenuItem helpMenuItem;
@@ -53,6 +60,7 @@ public class A3Panel extends JPanel implements ActionListener, ChangeListener {
     boolean headLight = true;
     A3PHtmlViewer htmlViewer;
     A3PHtmlViewer helpViewer;
+    Action3D measure;
 
     Action3D a3;
     String actionName = "0";
@@ -85,28 +93,45 @@ public class A3Panel extends JPanel implements ActionListener, ChangeListener {
         headLightMenuItem = new JMenuItem("HeadLight");
         headLightMenuItem.addActionListener(this);
         configMenu.add(headLightMenuItem);
-
+        balloonMenu = new JMenu("Balloon");
+        configMenu.add(balloonMenu);
         balloonRightMenuItem = new JMenuItem("Balloon on (right)");
         balloonRightMenuItem.addActionListener(this);
-        configMenu.add(balloonRightMenuItem);
+        balloonMenu.add(balloonRightMenuItem);
         balloonLeftMenuItem = new JMenuItem("Balloon on (left)");
         balloonLeftMenuItem.addActionListener(this);
-        configMenu.add(balloonLeftMenuItem);
+        balloonMenu.add(balloonLeftMenuItem);
         balloonTopMenuItem = new JMenuItem("Balloon on (top)");
         balloonTopMenuItem.addActionListener(this);
-        configMenu.add(balloonTopMenuItem);
+        balloonMenu.add(balloonTopMenuItem);
         balloonBottomMenuItem = new JMenuItem("Balloon on (bottom)");
         balloonBottomMenuItem.addActionListener(this);
-        configMenu.add(balloonBottomMenuItem);
+        balloonMenu.add(balloonBottomMenuItem);
         balloonDelMenuItem = new JMenuItem("Balloon off");
         balloonDelMenuItem.addActionListener(this);
-        configMenu.add(balloonDelMenuItem);
+        balloonMenu.add(balloonDelMenuItem);
+        labelMenu = new JMenu("Label");
+        configMenu.add(labelMenu);
         labelMenuItem = new JMenuItem("Label on");
         labelMenuItem.addActionListener(this);
-        configMenu.add(labelMenuItem);
+        labelMenu.add(labelMenuItem);
         labelDelMenuItem = new JMenuItem("Label off");
         labelDelMenuItem.addActionListener(this);
-        configMenu.add(labelDelMenuItem);
+        labelMenu.add(labelDelMenuItem);
+        measureMenu = new JMenu("measure");
+        configMenu.add(measureMenu);
+        measureXZMenuItem = new JMenuItem("measure XZ");
+        measureXZMenuItem.addActionListener(this);
+        measureMenu.add(measureXZMenuItem);
+        measureXYMenuItem = new JMenuItem("measure XY");
+        measureXYMenuItem.addActionListener(this);
+        measureMenu.add(measureXYMenuItem);
+        measureYZMenuItem = new JMenuItem("measure YZ");
+        measureYZMenuItem.addActionListener(this);
+        measureMenu.add(measureYZMenuItem);
+        measureNoneMenuItem = new JMenuItem("measure None");
+        measureNoneMenuItem.addActionListener(this);
+        measureMenu.add(measureNoneMenuItem);
         htmlMenuItem = new JMenuItem("Html");
         htmlMenuItem.addActionListener(this);
         configMenu.add(htmlMenuItem);
@@ -121,15 +146,16 @@ public class A3Panel extends JPanel implements ActionListener, ChangeListener {
         configMenu.add(aboutMenuItem);
 
         consoleFrame = new JFrame("Console");
-        Box b = Box.createVerticalBox();
+        MyVerticalBox b = new MyVerticalBox();
         consoleTextArea = new JTextArea(20,60);
         consoleTextArea.setEditable(false);
         JScrollPane sp = new JScrollPane(consoleTextArea);
-        b.add(sp);
+        b.myAdd(sp,1.0);
         clearConsoleButton = new JButton("Clear");
         clearConsoleButton.addActionListener(this);
-        b.add(clearConsoleButton);
-        consoleFrame.getContentPane().add(b);
+        b.myAdd(clearConsoleButton,0.0);
+        consoleFrame.getContentPane().setLayout(new BorderLayout());
+        consoleFrame.getContentPane().add(b,BorderLayout.CENTER);
         consoleFrame.addWindowListener(new WindowAdapter() {
             public void windowClosing(WindowEvent we) {
                 consoleFrame.setVisible(false);
@@ -185,6 +211,12 @@ public class A3Panel extends JPanel implements ActionListener, ChangeListener {
         htmlViewer = new A3PHtmlViewer(frame);
         helpViewer = new A3PHtmlViewer(null);
 
+        try {
+            measure = new Action3D("x-res:///jp/sourceforge/acerola3d/a3viewer/resources/measure.a3");
+        } catch (Exception e) {
+            e.printStackTrace();
+        }
+
         Runnable r = new Runnable() {
             public void run() {
                 loadAction3D();
@@ -222,25 +254,25 @@ public class A3Panel extends JPanel implements ActionListener, ChangeListener {
 sp.setPreferredSize(new Dimension(50,150));
         controlBox.myAdd(sp,1.0);
 
-        Box volBox = Box.createHorizontalBox();
+        MyHorizontalBox volBox = new MyHorizontalBox();
         controlBox.myAdd(volBox,0.0);
-        volBox.add(new JLabel("SoundVol"));
+        volBox.myAdd(new JLabel("SoundVol"),0.0);
         volumeSlider = new JSlider();
         volumeSlider.setMinimum(0);
         volumeSlider.setMaximum(sliderLength);
         volumeSlider.addChangeListener(this);
-        volBox.add(volumeSlider);
+        volBox.myAdd(volumeSlider,1.0);
 
-        String buildTime = null;
+        String buildDate = null;
         try {
-            URL url = getClass().getClassLoader().getResource("buildTime.txt");
+            URL url = getClass().getClassLoader().getResource("buildDate.txt");
             InputStreamReader isr = new InputStreamReader(url.openStream(),"UTF-8");
             BufferedReader br = new BufferedReader(isr);
-            buildTime = br.readLine();
+            buildDate = br.readLine();
         } catch (Exception e) {
-            buildTime = "???";
+            buildDate = "???";
         }
-        controlBox.myAdd(new JLabel("build time:"+buildTime),0.0);
+        controlBox.myAdd(new JLabel("build date: "+buildDate),0.0);
 
         return controlBox;
     }
@@ -315,6 +347,17 @@ return;
             if (a3==null)
                 return;
             a3.setLabel(null);
+        } else if (ae.getSource() == measureXZMenuItem) {
+            measure.setRot(-Math.PI/2.0,0.0,0.0);
+            a3canvas.add(measure);
+        } else if (ae.getSource() == measureXYMenuItem) {
+            measure.setRot(0.0,0.0,0.0);
+            a3canvas.add(measure);
+        } else if (ae.getSource() == measureYZMenuItem) {
+            measure.setRot(0.0,Math.PI/2.0,0.0);
+            a3canvas.add(measure);
+        } else if (ae.getSource() == measureNoneMenuItem) {
+            a3canvas.del(measure);
         } else if (ae.getSource() == htmlMenuItem){
             openHtml();
         } else if (ae.getSource() == metadataMenuItem){
@@ -435,6 +478,15 @@ return;
 
         statusBarLabel.setText("Status: Loading ... done.");
     }
+    void loadAction3D(File f) throws Exception {
+        urlTextField.setText(f.toURI().toURL().toString());
+        Runnable r = new Runnable() {
+            public void run() {
+                loadAction3D();
+            }
+        };
+        executor.schedule(r,100,TimeUnit.MILLISECONDS);
+    }
 
     public void repaint() {
         if (a3 == null)
